import { Module } from '@nestjs/common';
import { AuthController } from './infrastructure/auth.controller';
import { LoginManager } from './domain/managers/login.manager';
import { LogoutManager } from './domain/managers/logout.manager';
import { AuthOrchestrator } from './domain/auth.orchestrator';
import { ConfigModule } from '@nestjs/config';
import { TypeOrmModule } from '@nestjs/typeorm';
import { CqrsModule } from '@nestjs/cqrs';
import { UserModel } from 'src/modules/user-related/user/data/models/user.model';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';
import { UserDataService } from 'src/modules/user-related/user/data/services/user-data.service';

@Module({
  imports: [
    ConfigModule.forRoot(),
    TypeOrmModule.forFeature([UserModel], CONNECTION_NAME.DEFAULT),
    CqrsModule,
  ],
  controllers: [AuthController],
  providers: [LoginManager, LogoutManager, UserDataService, AuthOrchestrator],
})
export class AuthModule {}
