import { Injectable } from '@nestjs/common';
import { CreateItemCategoryManager } from './managers/create-item-category.manager';
import { ItemCategoryDataService } from '../../data/services/item-category-data.service';
import { ItemCategoryEntity } from '../entities/item-category.entity';
import { DeleteItemCategoryManager } from './managers/delete-item-category.manager';
import { UpdateItemCategoryManager } from './managers/update-item-category.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveItemCategoryManager } from './managers/active-item-category.manager';
import { InactiveItemCategoryManager } from './managers/inactive-item-category.manager';
import { ConfirmItemCategoryManager } from './managers/confirm-item-category.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmItemCategoryManager } from './managers/batch-confirm-item-category.manager';
import { BatchInactiveItemCategoryManager } from './managers/batch-inactive-item-category.manager';
import { BatchActiveItemCategoryManager } from './managers/batch-active-item-category.manager';
import { BatchDeleteItemCategoryManager } from './managers/batch-delete-item-category.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class ItemCategoryDataOrchestrator extends BaseDataTransactionOrchestrator<ItemCategoryEntity> {
  constructor(
    private createManager: CreateItemCategoryManager,
    private updateManager: UpdateItemCategoryManager,
    private deleteManager: DeleteItemCategoryManager,
    private activeManager: ActiveItemCategoryManager,
    private confirmManager: ConfirmItemCategoryManager,
    private inactiveManager: InactiveItemCategoryManager,
    private batchDeleteManager: BatchDeleteItemCategoryManager,
    private batchActiveManager: BatchActiveItemCategoryManager,
    private batchConfirmManager: BatchConfirmItemCategoryManager,
    private batchInactiveManager: BatchInactiveItemCategoryManager,
    private serviceData: ItemCategoryDataService,
  ) {
    super();
  }

  async create(data): Promise<ItemCategoryEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.ITEM_CATEGORY);
    await this.createManager.execute();
    await this.createManager.generateConfig();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<ItemCategoryEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.ITEM_CATEGORY);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<String> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.ITEM_CATEGORY);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(
      this.serviceData,
      TABLE_NAME.ITEM_CATEGORY,
    );
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<String> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.ITEM_CATEGORY);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(
      this.serviceData,
      TABLE_NAME.ITEM_CATEGORY,
    );
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<String> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.ITEM_CATEGORY);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(
      this.serviceData,
      TABLE_NAME.ITEM_CATEGORY,
    );
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<String> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.ITEM_CATEGORY);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(
      this.serviceData,
      TABLE_NAME.ITEM_CATEGORY,
    );
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
