import { Module } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { TypeOrmModule } from '@nestjs/typeorm';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';
import { ProfitShareFormulaReadController } from './infrastructure/profit-share-formula-read.controller';
import { ProfitShareFormulaReadOrchestrator } from './domain/usecases/profit-share-formula-read.orchestrator';
import { ProfitShareFormulaDataController } from './infrastructure/profit-share-formula-data.controller';
import { ProfitShareFormulaDataOrchestrator } from './domain/usecases/profit-share-formula-data.orchestrator';
import { CqrsModule } from '@nestjs/cqrs';
import { UpdateProfitShareFormulaManager } from './domain/usecases/managers/update-profit-share-formula.manager';
import { DetailProfitShareFormulaManager } from './domain/usecases/managers/detail-profit-share-formula.manager';
import { SalesPriceFormulaModel } from '../sales-price-formula/data/models/sales-price-formula.model';

@Module({
  imports: [
    ConfigModule.forRoot(),
    TypeOrmModule.forFeature([SalesPriceFormulaModel], CONNECTION_NAME.DEFAULT),
    CqrsModule,
  ],
  controllers: [
    ProfitShareFormulaDataController,
    ProfitShareFormulaReadController,
  ],
  providers: [
    DetailProfitShareFormulaManager,
    UpdateProfitShareFormulaManager,

    ProfitShareFormulaDataOrchestrator,
    ProfitShareFormulaReadOrchestrator,
  ],
})
export class ProfitShareFormulaModule {}
