import { Injectable } from '@nestjs/common';
import { BaseDetailManager } from 'src/core/modules/domain/usecase/managers/base-detail.manager';
import { TransactionEntity } from '../../entities/transaction.entity';
import { RelationParam } from 'src/core/modules/domain/entities/base-filter.entity';

@Injectable()
export class DetailTransactionManager extends BaseDetailManager<TransactionEntity> {
  async prepareData(): Promise<void> {
    return;
  }

  async beforeProcess(): Promise<void> {
    return;
  }

  async afterProcess(): Promise<void> {
    let payment_type_bank;

    const season_period = {
      id: this.result.season_period_id,
      holiday_name: this.result.season_period_name,
      season_type: {
        id: this.result.season_period_type_id,
        name: this.result.season_period_type_name,
      },
    };

    if (this.result.payment_type_method_id) {
      payment_type_bank = {
        id: this.result.payment_type_method_id,
        issuer_name: this.result.payment_type_method_name,
        account_number: this.result.payment_type_method_number,
      };
    }

    const items = this.result?.['items']?.map((itemData) => {
      let tenant;

      if (itemData.item_tenant_id) {
        tenant = {
          id: itemData.item_tenant_id,
          name: itemData.item_tenant_name,
          share_margin: itemData.item_tenant_share_margin,
        };
      }

      return {
        item: {
          id: itemData.item_id,
          name: itemData.item_name,
          item_type: itemData.item_type,
          base_price: itemData.item_price,
          hpp: itemData.item_hpp,
          tenant: tenant,
          item_category: {
            id: itemData.item_category_id,
            name: itemData.item_category_name,
          },
        },
        id: itemData.item_category_id,
        name: itemData.item_category_name,
      };
    });

    Object.assign(this.result, {
      season_period: season_period,
      items: items,
      payment_type_bank: payment_type_bank,
    });

    delete this.result.season_period_id;
    delete this.result.season_period_name;
    delete this.result.season_period_type_id;
    delete this.result.season_period_type_name;

    delete this.result.payment_type_method_id;
    delete this.result.payment_type_method_name;
    delete this.result.payment_type_method_number;

    return;
  }

  get relations(): RelationParam {
    return {
      // relation only join (for query purpose)
      joinRelations: [],

      // relation join and select (relasi yang ingin ditampilkan),
      selectRelations: ['items'],

      // relation yang hanya ingin dihitung (akan return number)
      countRelations: [],
    };
  }

  get selects(): string[] {
    return [
      `${this.tableName}.id`,
      `${this.tableName}.creator_name`,
      `${this.tableName}.created_at`,
      `${this.tableName}.updated_at`,
      `${this.tableName}.editor_name`,
      `${this.tableName}.invoice_code`,

      `${this.tableName}.season_period_id`,
      `${this.tableName}.season_period_name`,
      `${this.tableName}.season_period_type_id`,
      `${this.tableName}.season_period_type_name`,

      `${this.tableName}.status`,
      `${this.tableName}.no_of_group`,
      `${this.tableName}.customer_type`,
      `${this.tableName}.customer_name`,
      `${this.tableName}.customer_phone`,
      `${this.tableName}.customer_email`,
      `${this.tableName}.customer_description`,
      `${this.tableName}.booking_date`,

      `${this.tableName}.discount_percentage`,
      `${this.tableName}.discount_value`,
      `${this.tableName}.payment_type`,
      `${this.tableName}.payment_date`,
      `${this.tableName}.payment_total_pay`,
      `${this.tableName}.payment_type_method_id`,
      `${this.tableName}.payment_type_method_name`,
      `${this.tableName}.payment_type_method_number`,

      `${this.tableName}.payment_sub_total`,
      `${this.tableName}.payment_discount_total`,
      `${this.tableName}.payment_total`,

      'items',
    ];
  }

  get setFindProperties(): any {
    return {
      id: this.dataId,
    };
  }
}
