import { Injectable } from '@nestjs/common';
import { CreateVipCategoryManager } from './managers/create-vip-category.manager';
import { VipCategoryDataService } from '../../data/services/vip-category-data.service';
import { VipCategoryEntity } from '../entities/vip-category.entity';
import { DeleteVipCategoryManager } from './managers/delete-vip-category.manager';
import { UpdateVipCategoryManager } from './managers/update-vip-category.manager';
import { BaseDataTransactionOrchestrator } from 'src/core/modules/domain/usecase/orchestrators/base-data-transaction.orchestrator';
import { ActiveVipCategoryManager } from './managers/active-vip-category.manager';
import { InactiveVipCategoryManager } from './managers/inactive-vip-category.manager';
import { ConfirmVipCategoryManager } from './managers/confirm-vip-category.manager';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { BatchResult } from 'src/core/response/domain/ok-response.interface';
import { BatchConfirmVipCategoryManager } from './managers/batch-confirm-vip-category.manager';
import { BatchInactiveVipCategoryManager } from './managers/batch-inactive-vip-category.manager';
import { BatchActiveVipCategoryManager } from './managers/batch-active-vip-category.manager';
import { BatchDeleteVipCategoryManager } from './managers/batch-delete-vip-category.manager';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';

@Injectable()
export class VipCategoryDataOrchestrator extends BaseDataTransactionOrchestrator<VipCategoryEntity> {
  constructor(
    private createManager: CreateVipCategoryManager,
    private updateManager: UpdateVipCategoryManager,
    private deleteManager: DeleteVipCategoryManager,
    private activeManager: ActiveVipCategoryManager,
    private confirmManager: ConfirmVipCategoryManager,
    private inactiveManager: InactiveVipCategoryManager,
    private batchDeleteManager: BatchDeleteVipCategoryManager,
    private batchActiveManager: BatchActiveVipCategoryManager,
    private batchConfirmManager: BatchConfirmVipCategoryManager,
    private batchInactiveManager: BatchInactiveVipCategoryManager,
    private serviceData: VipCategoryDataService,
  ) {
    super();
  }

  async create(data): Promise<VipCategoryEntity> {
    this.createManager.setData(data);
    this.createManager.setService(this.serviceData, TABLE_NAME.VIP_CATEGORY);
    await this.createManager.execute();
    await this.createManager.generateConfig();
    return this.createManager.getResult();
  }

  async update(dataId, data): Promise<VipCategoryEntity> {
    this.updateManager.setData(dataId, data);
    this.updateManager.setService(this.serviceData, TABLE_NAME.VIP_CATEGORY);
    await this.updateManager.execute();
    return this.updateManager.getResult();
  }

  async delete(dataId): Promise<String> {
    this.deleteManager.setData(dataId);
    this.deleteManager.setService(this.serviceData, TABLE_NAME.VIP_CATEGORY);
    await this.deleteManager.execute();
    return this.deleteManager.getResult();
  }

  async batchDelete(dataIds: string[]): Promise<BatchResult> {
    this.batchDeleteManager.setData(dataIds);
    this.batchDeleteManager.setService(
      this.serviceData,
      TABLE_NAME.VIP_CATEGORY,
    );
    await this.batchDeleteManager.execute();
    return this.batchDeleteManager.getResult();
  }

  async active(dataId): Promise<String> {
    this.activeManager.setData(dataId, STATUS.ACTIVE);
    this.activeManager.setService(this.serviceData, TABLE_NAME.VIP_CATEGORY);
    await this.activeManager.execute();
    return this.activeManager.getResult();
  }

  async batchActive(dataIds: string[]): Promise<BatchResult> {
    this.batchActiveManager.setData(dataIds, STATUS.ACTIVE);
    this.batchActiveManager.setService(
      this.serviceData,
      TABLE_NAME.VIP_CATEGORY,
    );
    await this.batchActiveManager.execute();
    return this.batchActiveManager.getResult();
  }

  async confirm(dataId): Promise<String> {
    this.confirmManager.setData(dataId, STATUS.ACTIVE);
    this.confirmManager.setService(this.serviceData, TABLE_NAME.VIP_CATEGORY);
    await this.confirmManager.execute();
    return this.confirmManager.getResult();
  }

  async batchConfirm(dataIds: string[]): Promise<BatchResult> {
    this.batchConfirmManager.setData(dataIds, STATUS.ACTIVE);
    this.batchConfirmManager.setService(
      this.serviceData,
      TABLE_NAME.VIP_CATEGORY,
    );
    await this.batchConfirmManager.execute();
    return this.batchConfirmManager.getResult();
  }

  async inactive(dataId): Promise<String> {
    this.inactiveManager.setData(dataId, STATUS.INACTIVE);
    this.inactiveManager.setService(this.serviceData, TABLE_NAME.VIP_CATEGORY);
    await this.inactiveManager.execute();
    return this.inactiveManager.getResult();
  }

  async batchInactive(dataIds: string[]): Promise<BatchResult> {
    this.batchInactiveManager.setData(dataIds, STATUS.INACTIVE);
    this.batchInactiveManager.setService(
      this.serviceData,
      TABLE_NAME.VIP_CATEGORY,
    );
    await this.batchInactiveManager.execute();
    return this.batchInactiveManager.getResult();
  }
}
