import { Injectable } from '@nestjs/common';
import { BaseIndexManager } from 'src/core/modules/domain/usecase/managers/base-index.manager';
import { UserPrivilegeConfigurationEntity } from '../../../entities/user-privilege-configuration.entity';
import { SelectQueryBuilder } from 'typeorm';
import {
  Param,
  RelationParam,
} from 'src/core/modules/domain/entities/base-filter.entity';
import { ORDER_TYPE } from 'src/core/strings/constants/base.constants';

@Injectable()
export class IndexUserPrivilegeConfigurationManager extends BaseIndexManager<UserPrivilegeConfigurationEntity> {
  async prepareData(): Promise<void> {
    Object.assign(this.filterParam, {
      order_by: this.filterParam.order_by ?? `${this.tableName}.index`,
      order_type: this.filterParam.order_by ?? ORDER_TYPE.ASC,
      limit: 100,
    });
    return;
  }

  async beforeProcess(): Promise<void> {
    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  get relations(): RelationParam {
    return {
      joinRelations: [],
      selectRelations: [],
      countRelations: [],
    };
  }

  get selects(): string[] {
    return [];
  }

  get specificFilter(): Param[] {
    return [];
  }

  setQueryFilter(
    queryBuilder: SelectQueryBuilder<UserPrivilegeConfigurationEntity>,
  ): SelectQueryBuilder<UserPrivilegeConfigurationEntity> {
    if (this.filterParam.modules) {
      queryBuilder.andWhere('module In (:...modules)', {
        modules: this.filterParam.modules,
      });
    }

    if (this.filterParam.menus) {
      queryBuilder.andWhere('menu In (:...menus)', {
        menus: this.filterParam.menus,
      });
    }

    if (this.filterParam.user_privilege_ids) {
      queryBuilder.andWhere(`user_privilege_id In (:...privilegeIds)`, {
        privilegeIds: this.filterParam.user_privilege_ids,
      });
    }

    return queryBuilder;
  }
}
